% P32NKQ.m NKQ
% COPYRIGHT:  Aprend Technology, 2012. All rights reserved.
 
%% init vars
post_title = 'P32NKQ';
dirname = ['G:\Projects\Blog\Posts\' post_title '\'];
addpath(dirname);
addpath('G:\Projects\DEbook\NoiseCalcs');

%% generate photon energy spectrum and inverse of cumulative 
  % will be used to generate random numbers representing photon energy
% [spec,egys] = XrayTubeSpectrum(120,1,'no_characteristic_radiation'); % 120 kV, 1 keV step
[specnum,egys] = XrayTubeSpectrumTasmip(120,'number_spectrum','clipzeros'); % 120 kV, 1 keV step
spec = specnum.*egys;
specn = spec(:)/sum(spec(:));
specum = cumsum(specn(:));
         % invert cumsum using interp1,          
specuminv = interp1(specum,egys,linspace(0,1,1000),'linear','extrap'); 
assert(~any(isnan(specuminv(:))));
    % compute the ebar and e2bar over the normalized energy region
specnn = specn/trapz(egys,specn);
Ebar = trapz(egys,egys(:).*specnn(:));
E2bar = trapz(egys,(egys(:).^2).*specnn(:));
F = E2bar/Ebar^2;

%% stats vs. Ethreshold
lambda = 1000;
ntrials = 50000;
Ethresholds = linspace(40,75,5);
nthresh = numel(Ethresholds);
C2thry_all = zeros(3,3,nthresh);
C2MC_all = zeros(3,3,nthresh); 
fprintf('\nStarting Monte Carlo--this will take a while\n');
for kthresh = 1:nthresh
    fprintf('*');
   E_threshold = Ethresholds(kthresh);
      % get MC data
   stats = N2Qsim(specuminv,lambda,E_threshold,ntrials);
       % theoretical results
   [minval,kthreshold] = min(abs(egys - E_threshold));
    egys1 = egys(1:kthreshold);
    spec1 = specnn(1:kthreshold);
    lambdas(1) = lambda*trapz(egys1,spec1);
    Ebars(1) = trapz(egys1,egys1(:).*spec1(:))/trapz(egys1,spec1);
    
    egys2 = egys(kthreshold+1:end);
    spec2 = specnn(kthreshold+1:end);
    lambdas(2) = lambda*trapz(egys2,spec2);
    Ebars(2) = trapz(egys2,egys2(:).*spec2(:))/trapz(egys2,spec2);
    
    C2thry = [lambdas(1) 0 lambdas(1)*Ebars(1); ...
                   0  lambdas(2) lambdas(2)*Ebars(2); ...
                   lambdas(1)*Ebars(1) lambdas(2)*Ebars(2) lambda*E2bar];
    C2thry_all(:,:,kthresh) = C2thry;
    
    C2MC = cov(stats.dat(:,2:4));
    C2MC_all(:,:,kthresh) = C2MC;
end
fprintf('\n');

%% plot covariance and correlation data
linewid = 1.5;
fh = figure;
subplot(121)
clear hs
hs(1) = plot(Ethresholds,squeeze(C2thry_all(1,3,:)),'-r','linewidth',linewid);
hold on,plot(Ethresholds,squeeze(C2MC_all(1,3,:)),'or'),hold off
hold on,hs(2) = plot(Ethresholds,squeeze(C2thry_all(2,3,:)),'-k','linewidth',linewid);hold off
hold on,plot(Ethresholds,squeeze(C2MC_all(2,3,:)),'+k'),hold off
axis([30 80 0 1e5])
legend(hs,'cov(N1,Q)','cov(N2,Q)');
xlabel('threshold energy (keV) ','fontsize',14)
ylabel('covariance','fontsize',14)
set(gca,'linewidth',1.5,'fontsize',14)

   % plot correlations
   % convert covariances to correlations
crs_thry = zeros(size(C2thry_all));   
crs_MC = zeros(size(C2thry_all));   
for k = 1: nthresh
    crs_thry(:,:,k) = cov2corr(C2thry_all(:,:,k));
    crs_MC(:,:,k) = cov2corr(C2MC_all(:,:,k));
end
subplot(122)
clear hs
hs(1) = plot(Ethresholds,squeeze(crs_thry(1,3,:)),'-r','linewidth',linewid);
hold on,plot(Ethresholds,squeeze(crs_MC(1,3,:)),'or'),hold off
hold on,hs(2) = plot(Ethresholds,squeeze(crs_thry(2,3,:)),'-k','linewidth',linewid);hold off
hold on,plot(Ethresholds,squeeze(crs_MC(2,3,:)),'+k'),hold off
axis([30 80 0 1.5])
legend(hs,'corr(N1,Q)','corr(N2,Q)');
xlabel('threshold energy (keV) ','fontsize',14)
ylabel('correlation','fontsize',14)
set(gca,'linewidth',1.5,'fontsize',14,'Yticklabel',{'0','0.5','1',''})
psn = [250 100 850 450];
set(fh,'color','w','position',psn)


%% export the figure as png and eps
figno = sprintf('-f%d',fh);
fname = 'N2Qstats';
   % export as eps
exportfig(fh,[dirname fname '.eps'],'width',6,'LockAxes',0,'color','cmyk');
set(fh,'papertype','usletter')
print(figno,'-dpng','-cmyk','-r100',[dirname fname '.png']) % resolution 100 dpi
